Attribute VB_Name = "ExtractShapeInfo"
#If VBA7 Then
    Declare PtrSafe Function OpenClipboard Lib "user32" (ByVal hWnd As LongPtr) As Long
    Declare PtrSafe Function CloseClipboard Lib "user32" () As Long
    Declare PtrSafe Function EmptyClipboard Lib "user32" () As Long
    Declare PtrSafe Function SetClipboardData Lib "user32" (ByVal uFormat As Long, ByVal hMem As LongPtr) As LongPtr
    Declare PtrSafe Function GlobalAlloc Lib "kernel32" (ByVal uFlags As Long, ByVal dwBytes As LongPtr) As LongPtr
    Declare PtrSafe Function GlobalLock Lib "kernel32" (ByVal hMem As LongPtr) As LongPtr
    Declare PtrSafe Function GlobalUnlock Lib "kernel32" (ByVal hMem As LongPtr) As Long
    Declare PtrSafe Function lstrcpy Lib "kernel32" (ByVal lpString1 As Any, ByVal lpString2 As Any) As LongPtr
#Else
    Declare Function OpenClipboard Lib "user32" (ByVal hWnd As Long) As Long
    Declare Function CloseClipboard Lib "user32" () As Long
    Declare Function EmptyClipboard Lib "user32" () As Long
    Declare Function SetClipboardData Lib "user32" (ByVal uFormat As Long, ByVal hMem As Long) As Long
    Declare Function GlobalAlloc Lib "kernel32" (ByVal uFlags As Long, ByVal dwBytes As Long) As Long
    Declare Function GlobalLock Lib "kernel32" (ByVal hMem As Long) As Long
    Declare Function GlobalUnlock Lib "kernel32" (ByVal hMem As Long) As Long
    Declare Function lstrcpy Lib "kernel32" (ByVal lpString1 As Any, ByVal lpString2 As Any) As Long
#End If

Const GHND As Long = &H42
Const CF_TEXT As Long = 1
Const MEM_RELEASE As Long = &H8000

Sub ExtractShapeInfo()
    Dim shp As Shape
    Dim slide As slide
    Dim dataString As String
    Dim shapeType As String
    Dim posX As Single, posY As Single ' Variables for position
    Dim hGlobalMemory As LongPtr, lpGlobalMemory As LongPtr
    Dim lngSize As Long
    
    Set slide = ActivePresentation.Slides(ActiveWindow.View.slide.SlideIndex)
    dataString = "Shape Type,Position X (in),Position Y (in),Width (in),Height (in)" & vbCrLf
    
    For Each shp In slide.Shapes
        shapeType = GetShapeType(shp)
        
        ' Adjust position for circles to be the center
        If shapeType = "Circle" Then
            posX = (shp.Left + shp.Width / 2) / 72
            posY = (shp.Top + shp.Height / 2) / 72
        Else
            posX = shp.Left / 72
            posY = shp.Top / 72
        End If
        
        With shp
            dataString = dataString & shapeType & "," & _
                          Format(posX, "0.00") & "," & _
                          Format(posY, "0.00") & "," & _
                          Format(.Width / 72, "0.00") & "," & _
                          Format(.Height / 72, "0.00") & vbCrLf
        End With
    Next shp
    
    ' Allocate memory and copy the string to it
    lngSize = Len(dataString) + 1
    hGlobalMemory = GlobalAlloc(GHND, lngSize)
    lpGlobalMemory = GlobalLock(hGlobalMemory)
    lstrcpy ByVal lpGlobalMemory, dataString
    GlobalUnlock hGlobalMemory
    
    ' Open Clipboard and set the data
    If OpenClipboard(0&) Then
        Call EmptyClipboard
        Call SetClipboardData(CF_TEXT, hGlobalMemory)
        Call CloseClipboard
    End If
    
    MsgBox "Shape information has been copied to the clipboard in inches, with circles' positions as their centers.", vbInformation
End Sub

Function GetShapeType(shp As Shape) As String
    On Error Resume Next ' In case of a shape type that does not have an AutoShapeType property
    Select Case shp.AutoShapeType
        Case msoShapeOval
            GetShapeType = "Circle"
        Case msoShapeRectangle
            GetShapeType = "Rectangle"
        Case Else
            If shp.Type = msoAutoShape Then
                GetShapeType = "Other AutoShape"
            ElseIf shp.Type = msoLine Then
                GetShapeType = "Line"
            ElseIf shp.Type = msoFreeform Then
                GetShapeType = "Freeform"
            Else
                GetShapeType = "Other"
            End If
    End Select
    On Error GoTo 0 ' Turn back on regular error handling
End Function


